package com.fjhx.customer.service.customer.impl;

import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.fjhx.area.utils.AreaUtil;
import com.fjhx.customer.entity.customer.dto.CustomerUserDto;
import com.fjhx.customer.entity.customer.po.Customer;
import com.fjhx.customer.entity.customer.po.CustomerUser;
import com.fjhx.customer.mapper.customer.CustomerMapper;
import com.fjhx.customer.service.customer.CustomerService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.customer.service.customer.CustomerUserService;
import com.fjhx.customer.utils.code.CodeEnum;
import com.obs.services.internal.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.customer.entity.customer.vo.CustomerVo;
import com.fjhx.customer.entity.customer.dto.CustomerSelectDto;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.fjhx.customer.entity.customer.dto.CustomerDto;
import cn.hutool.core.bean.BeanUtil;
import org.springframework.transaction.annotation.Transactional;

import java.rmi.ServerException;
import java.security.Security;
import java.util.List;

/**
 * <p>
 * 客户表 服务实现类
 * </p>
 *
 * @author
 * @since 2023-04-04
 */
@Service
public class CustomerServiceImpl extends ServiceImpl<CustomerMapper, Customer> implements CustomerService {
    @Autowired
    private CustomerUserService customerUserService;


    /**
     * 查询客户的列表
     * @param dto
     * @return
     */
    @Override
    public Page<CustomerVo> getPage(CustomerSelectDto dto) {
        LambdaQueryWrapper<Customer> wrapper = Wrappers.<Customer>lambdaQuery();
        wrapper.eq(ObjectUtil.isNotEmpty(dto.getSource()),Customer::getSource,dto.getSource());
        wrapper.eq(ObjectUtil.isNotEmpty(dto.getStatus()),Customer::getStatus,dto.getStatus());
        //客户状态（0：公海客户  1：私海客户）(业务员ID为null为公海客户，业务员ID不为null为私海客户)
        if (ObjectUtil.isNotEmpty(dto.getType())){
            //私海客户查询
            if (dto.getType().equals("1")){
                wrapper.isNotNull(Customer::getUserId);
            }else if (dto.getType().equals("0")){
                wrapper.and(wrapper1 -> wrapper1.isNull(Customer::getUserId).or().eq(Customer::getUserId,""));
            }
        }
        if (ObjectUtil.isNotEmpty(dto.getKeyword())){
            //查询客户名称或者客户编码
            wrapper.and(wrapper1 -> wrapper1.like(Customer::getName,dto.getKeyword()).or().eq(Customer::getCode,dto.getKeyword()));
        }
        wrapper.orderByDesc(Customer::getCreateTime);
        Page<CustomerVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);

        //复制城市信息
        List<CustomerVo> records = page.getRecords();
        AreaUtil.setAreaName(records);
        return page;
    }

    /**
     * 查询客户的详情
     * @param id
     * @return
     */
    @Override
    public CustomerVo detail(Long id) {
        //查询客户表的信息
        Customer Customer = this.getById(id);
        CustomerVo result = BeanUtil.toBean(Customer, CustomerVo.class);
        if (ObjectUtil.isEmpty(result)){
            throw new ServiceException("没有找到该用户信息");
        }
        //查询客户-联系人表的信息
        List<CustomerUser> customerUserList = customerUserService.list(Wrappers.<CustomerUser>lambdaQuery()
                .eq(CustomerUser::getCustomerId, result.getId()));
        result.setCustomerUserList(customerUserList);
        AreaUtil.setAreaName(result);
        return result;
    }

    /**
     * 添加客户表的数据
     * @param customerDto
     */
    @Override
    @Transactional(rollbackFor = {Exception.class})
    public void add(CustomerDto customerDto) {
        customerDto.setCode(CodeEnum.CUSTOMER.getCode());
        this.save(customerDto);
        //添加库户-联系人表的信息
        saveCustomerUse(customerDto);
    }

    /**
     * 修改客户表的数据
     * @param customerDto
     */
    @Transactional(rollbackFor = {Exception.class})
    @Override
    public void edit(CustomerDto customerDto) {
        //修改客户表的信息
        this.updateById(customerDto);
        //删除客户-用户表的信息
        customerUserService.remove(Wrappers.<CustomerUser>lambdaQuery().eq(CustomerUser::getCustomerId,customerDto.getId()));
        //添加客户-联系人表的信息
        saveCustomerUse(customerDto);
    }

    /**
     * 删除客户表的数据
     * @param
     */
    @Override
    @Transactional(rollbackFor = {Exception.class})
    public void delete(Long id) {
        //删除客户表的数据
        this.removeById(id);
        //删除客户-联系人表的数据
        customerUserService.remove(Wrappers.<CustomerUser>lambdaQuery().eq(CustomerUser::getCustomerId,id));
    }

    /**
     * 私海客户表分页（添加权限自己看自己）
     */
    @Override
    public Page<CustomerVo> privateSeaPage(CustomerSelectDto dto) {
        LambdaQueryWrapper<Customer> wrapper = Wrappers.<Customer>lambdaQuery();
        wrapper.eq(ObjectUtil.isNotEmpty(dto.getSource()),Customer::getSource,dto.getSource());
        wrapper.eq(ObjectUtil.isNotEmpty(dto.getStatus()),Customer::getStatus,dto.getStatus());
        if (ObjectUtil.isNotEmpty(dto.getKeyword())){
            //查询客户名称或者客户编码
            wrapper.and(wrapper1 -> wrapper1.like(Customer::getName,dto.getKeyword()).or().eq(Customer::getCode,dto.getKeyword()));
        }
        //添加权限（自己看自己）
        wrapper.eq(Customer::getUserId, SecurityUtils.getUserId());
        wrapper.orderByDesc(Customer::getCreateTime);
        Page<CustomerVo> page =baseMapper.getPage(dto.getPage(), wrapper);
        //复制城市信息
        List<CustomerVo> records = page.getRecords();
        AreaUtil.setAreaName(records);
        return page;
    }

    //添加客户用户表的信息
    private void  saveCustomerUse(CustomerDto customerDto){
        List<CustomerUser> customerUserDtoList = customerDto.getCustomerUserList();
        customerUserDtoList.forEach(customerUserDto -> customerUserDto.setCustomerId(customerDto.getId()));
        customerUserService.saveBatch(customerUserDtoList);
    }

}
